%% Core Vector Machine (with probabilistic speed up)
%
% Input:
%       fKernel   - pointer to a valid kernel function f(i,j)
%       L         - N x 1 - numeric, label vector 0/1
%       epsilon   - double  -- coreset tolerance
%
% Output:
%       mAlpha    - alpha matrix of the core points with respect to the kernel
%
% (C) 2014 by Frank-Michael Schleif
% schleif@informatik.uni-leipzig.de

function oModel = core_pvm(fKernelNormal,L,epsilon,varargin)
if(~(isreal(epsilon) && epsilon>0 && epsilon<=1))
	error('Invalid input parameters');
end

vUniqueL = unique(L);
if(~((vUniqueL(1)==-1 || vUniqueL(1)==1) && (vUniqueL(2)==-1 || vUniqueL(2)==1)))
    error('Invalid input labels');
end

C            = 10;
if(numel(varargin)>1)
        C= varargin{2};
end

% to have generalized CVM
%fKernel      = @(A,B)bsxfun(@times,L(A),L(B)') .* fKernelNormal(A,B) + bsxfun(@times,L(A),L(B)') + eye(numel(A),numel(B))/C;
fKernelHelper = @(LabelMap,A,B,L) LabelMap.* fKernelNormal(A,B) + LabelMap + bsxfun(@eq,A,B')'/C;
fKernel       = @(A,B)fKernelHelper(bsxfun(@times,L(A),L(B)'),A,B,L);

if(isempty(varargin) || isempty(varargin{1}))
    vDiags = GetDiags(fKernel,[1:numel(L)]);
else
    vDiags = varargin{1};
    vDiags = vDiags + 1/C+1+1;
    % modify diags to fit the generalized Kernel
end
[S0,R0,vAlpha0]   = core_set_one_two(fKernel,epsilon,vDiags); % using only the kernel
[CoreVectors,Ind] = sort(S0);
oModel.R           = R0.^2;
oModel.CoreVectors = CoreVectors;
oModel.Alphas      = vAlpha0(Ind);
oModel.bias        = sum(oModel.Alphas .* L(oModel.CoreVectors));
oModel.L           = L;

function Diags = GetDiags(fKernelPtr,indices)
    dlen   = numel(indices);    
    Diags  = zeros(1,dlen);
    if(dlen>1000)
        vBlocks = 1:1000:dlen;
        for(k=1:numel(vBlocks))
                iStart = vBlocks(k);
                iEnd   = vBlocks(min(k+1,end));
                vRange = iStart:iEnd;
                Diags(vRange) = arrayfun(@(x) fKernelPtr(x,x), vRange);    
        end        
        if(iEnd<dlen)
            vRange = iEnd+1:dlen;
            Diags(vRange) = arrayfun(@(x) fKernelPtr(x,x), vRange);    
        end
    else
        Diags = arrayfun(@(x) fKernelPtr(x,x), indices);    
    end
    Median = median(Diags);
%    if(any(abs(Diags-Median) > 1e-4)) % - for generalized it should be ok now
%        error(sprintf('%s\n%s\n%s', 'Core Vector Machine only valid model for kernel with constant diagonal', 'you can correct the kernel e.g. as : K_diag_const = K./(sqrt(diag(K)) * sqrt(diag(K))'')', 'or use the KernelFilter method'));
%    end
    Diags(:) = Median;
